module.exports = function (grunt) {
	
	var backendModules = ['./gen', '../backend/vendor/teknisa/pebLibraries'];
	
	grunt.initConfig({
		buildEnvironment: {
			development: {
				configFile: "./parameters/devEnvironment.json"
			},
			production: {
				configFile: "./parameters/prodEnvironment.json"
			},
			test: {
				configFile: "./parameters/testEnvironment.json"
			},
			srcFolders: ["./config", "./reports", "../mobile", "../mobile/assets/json"]
		}
    });
    
    function buildFileContent(srcFiles, filePath, directory) {
		var parts = filePath.split('/');
		parts.shift();
		parts.shift();
		var filename = parts[parts.length - 1];
		var extension = filename.split('.').pop().toUpperCase();
		if (extension == 'JSON' || extension == 'XML') {
			if (directory == 'datasources') {
				filename = parts.pop();
				filename = filename.replace(/\.[^/.]+$/, "");
				srcFiles[directory][filename] = './../../../' + parts.join('/');
			} else {
				srcFiles[directory].push('/../../' + parts.join('/'));
			}
		}
	}
    
    grunt.registerTask('backendStep', 'build backend config files', function () {
		var srcFiles = {
			datasources: {}
		};
		var directories = ['datasources'];
		var dest = {
			datasources: '../backend/config/dataSourcesMapping.json'
		};
		directories.forEach(function (directory) {
			backendModules.forEach(function (module) {
				var moduleParts = module.split('/');
				var moduleName = moduleParts[moduleParts.length - 1].toUpperCase();
				var subModules = grunt.file.expand({
					filter: 'isDirectory'
				}, module + '/src/Teknisa/' + moduleName + '/Modules/*');

				subModules.forEach(function (subModule) {
					grunt.file.expand({
						filter: 'isFile'
					}, subModule + '/' + directory + '/**/*').forEach(function (filePath) {
						buildFileContent(srcFiles, filePath, directory);
					});
				});
				grunt.file.expand({
					filter: 'isFile'
				}, module + '/' + directory + '/**/*').forEach(function (filePath) {
					buildFileContent(srcFiles, filePath, directory);
				});
			});
		});
		grunt.file.write(dest.datasources, JSON.stringify(srcFiles.datasources, null, 2));
	});
	
	// task to build translate.properties
	grunt.registerTask('reportTranslate', 'build translate to BIRT reports', function () {
		var languages = ['en_us', 'es_cl', 'es_co', 'es_es'];
		var srcWords = "../mobile/assets/json/words/";
		var srcMessages = "../mobile/assets/json/";
		var srcReport = "reports/";
		var reportTrans = grunt.file.readJSON(srcMessages+"messages.json");
		
		languages.forEach( lang => {
			var words = grunt.file.readJSON(srcWords+lang+".json");
			var properties = '';

			Object.keys(reportTrans).forEach(keyRaiz => {
				if (typeof reportTrans[keyRaiz] === 'string') properties += `${keyRaiz}=${unicodeEscape(reportTrans[keyRaiz])}\n`;
				else {
					Object.keys(reportTrans[keyRaiz]).forEach(key => {
						properties += `${key}=${unicodeEscape(words[keyRaiz][key])}\n`;
					});
				}
			});
			
			grunt.file.write(srcReport+lang+".properties", properties);
		});
	});

	function unicodeEscape( stringToEscape ) {
		var escapedString = '';
		if(stringToEscape) {
			escapedString = stringToEscape.replace(/[^0-9|^a-z ]/gi, ( char ) => {
	          return `\\u${(`000${char.charCodeAt(0).toString(16)}`).substr(-4)}`;
	        });	
		}
		return escapedString;
	}
    
	grunt.loadNpmTasks('grunt-zh-environment-builder');
		
    grunt.registerTask('default', ['buildEnvironment', 'backendStep', 'reportTranslate']);
    
};