import {
  Loader, Metadata, Config, IEventParam, IDictionary,
} from '@zeedhi/core';
import {
  Form, Button, Select,
} from '@zeedhi/common';
import { PmeMask } from '@/config';
  
export class Pme03800HistoricoReqAnalitica {
  public reportController = Loader.getInstance('ReportController');

  public appController = Loader.getInstance('AppController');

  public utilities = Loader.getInstance('Utilities');
  
  public defaultReport({ component }: IEventParam<Select>) {
    component.setValue(component.datasource.data[0]);
  }

  public setFilterTpCardapio({ component }: IEventParam<Select>) {
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: null,
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public setFilterAgreg({ component }: IEventParam<Select>) {
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: 'T',
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('formHistorico');
    if (this.utilities.centralIsValid() && form.validate()) {
      const row: IDictionary = form.value;
      this.setParams(row);
      
      row.reportFormat = 'pdf';
      this.utilities.activeLoadingButton(component);
      this.reportController.generateReport('/report/HistoricoRequisicao', row, row.CD_REPORT_CODE).finally(() => {
        this.utilities.disableLoadingButton(component);
      });
    }
  }
  
  private setParams(row: IDictionary) {
    const mask: string = (Config.masks.productMask as PmeMask).pattern;
    row.CDFILCENTPME = this.appController.getCdCentral();
    row.NMFILCENTPME = this.appController.getNmCentral();
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
    row.PRODUCTMASK = this.utilities.formatReportMask(mask);
  }

  public clearFields({ component }: IEventParam<Select>) {
    const unidade = Metadata.getInstance<Select>('unidadeHistorico');
    const servico = Metadata.getInstance<Select>('servicoHistorico');

    if (component.name === 'tipoCardHistorico') {
      servico.setValue('');
      unidade.setValue('');
    } else unidade.setValue('');
  }
}
